;                                 REVLINE.LSP
;                                 Created By:
;
;                                David Hutchison
;                              HUTCHWARE Ltd.  1988
;
;  This routine is used to reconstruct  (revise) a polyline with a 
;  user defined line definition.  It is a supplement to the linetype
;  procedure already supplied with AutoCAD.  It differs in one
;  major way by allowing for the line definition to be completely
;  represented even if the distance of the defined line is greater
;  than the distance between the vertices of the polyline, thus
;  avoiding the dreaded continuous line occurrence.
;
;                         REVISED July 20, 1988
;
;  Revision :  added command to create layer "revline" thus allowing
;              reconstruction of the pline on a separate layer. This
;              in turn retains the integrity of the original pline
;              definition.  Also commented out the erase command
;              on the original pline. The end user can remove
;              the semi-colon to activate the command, however this
;              will remove the original pline. ( an undo will bring
;              it back ) Your choice..
;
;                        REVISED August 10, 1988
;
;  Revision :  added prompt for distance between dashes when no ( 0 ) dots
;              are requested.
;
;  Note on usage:  When the command "revline" is issued the prompt
;                  "Select Polyline to change" will appear, choose
;                  a polyline.  The program will determine if the entity
;                  selected is a polyline.
;
;                  The redefinition of the polyline is almost limitless.
;                  When prompted for the length of dash, enter a numeric
;                  value representative of the scale factor being used.
;                  If a distance of zero ( 0 ) is given, the redefinition
;                  will consist solely of dots ( i.e . . . ). When prompted
;                  for the number of dots in sequence, any number is valid.
;                  If a zero ( 0 ) is given the line will consist solely of
;                  dashes (i.e. - - - -).  When prompted for the distance
;                  between dots/dashes, give a numeric value.  If the number 
;                  of dots previously specified is zero ( 0 ), the distance
;                  between dots becomes the distance between dashes.
;
;                  This routine works on AutoCAD version 2.5 and up,
;                  including Release 9.
;
(vmon)
(defun C:REVLINE ( / ax ay ent type fil next point len dist
                   dist2 pt1 pt2 eod chgy chgx seglen ang ang$
                   add xoff yoff pt3x pt3y pt3 dtcount spc dotnum)
;
(command "LAYER" "N" "revline" "")      ;create layer for line redefinition
(setvar "CMDECHO" 0)
(prompt "\nSelect a polyline to revise :")
(setq ent nil)
(while (null ent)                       ; Select an entity
    (setq ent (entsel))
)
(setq ent (car ent))
(setq type (entget ent))              ; Get entity name
(IF (= "POLYLINE" (cdr (assoc '0 type)))
    (progn
       (setq layr (cdr (assoc '8 TYPE)))
       (command "LAYER" "S" "revline" "")
       (POLY )                        ; Entity is a poly line
    )
    (NOPOLY)                          ; Entity is not a poly line
)
)
;
;
;
(defun POLY ()
(princ "\nFinding all polyline vertices .... please wait..")
(terpri)
(setq fil (open  "TEMP.FIL" "w"))     ; Open temporary file
(setq next (entget (entnext ent)))    ; Get next entity
(while (/= (cdr (assoc '0 next)) "SEQEND")
     (if (and(= (cdr (assoc '0 next)) "VERTEX")     ; If entity is a vertex
	   (/= (cdr(assoc '70 next))16))
         (progn
             (setq point (cdr (assoc '10 next)))
             (setq ax (rtos (car point)))
             (setq ay (rtos (cadr point)))       ; Save coordinates to file
             (write-line ax fil)
             (write-line ay fil)
             (setq next (cdr (assoc '-1 next))) ; Get next entity
             (setq next (entget (entnext next)))
         )
         (progn
             (setq next (cdr (assoc '-1 next)))  ; Else just get next entity
             (setq next (entget (entnext next)))
         )
     )
)
(close fil)
(princ "\nSearch complete .................")
(terpri)
(LINEDEF)
;
;
;(command "ERASE" point "")              ; Erase selected polyline
                                         ; the semi-colon can be removed
                                         ;
(princ "\nRevising polyline with new definition ...please wait ..")
(setq fil (open "TEMP.FIL" "r"))
(setq dist 0.0)
(setq dist2 dist)
(setq ax (read-line fil))
(setq ay (read-line fil))
(setq ax (atof ax))
(setq ay (atof ay))
(setq pt1 (list ax ay))
(setq ax (read-line fil))            ;  Get beginning points from file
(setq ay (read-line fil))
(setq ax (atof ax))
(setq ay (atof ay))
(setq pt2 (list ax ay))
(setq eod "NO")
(setq switch 1)
(while (/= eod "YES")
      (if (= switch 1)
          (SEGMENT)                 ; Do line segment
          (DOT)                     ; Do dot segment
      )
)
(close fil)
(princ "\nRevision complete .............")
(terpri)
(command "LAYER" "S" layr "")       ; Restore layer to previous
(command "REDRAW")
)
;
(defun LINEDEF ()
;               Prompt for line definition
;
(setq len     (getreal "\nLength of dash ?..................< 1 > : "))
(setq dotnum  (getreal "\nNumber of dots in dot sequence ?..< 1 > : ")) 
(if (= dotnum 0)
    (setq spc (getreal "\nLength of space between dashes ?..< 3 > : "))
    (setq spc (getreal "\nLength of space between dots ?....< 3 > : "))
)
(if (= len nil) (setq len 1.0))
(if (= spc nil) (setq spc 1.0))    ; Set defaults
(if (= dotnum nil) (setq dotnum 3))
(if (or (and (= len 0) (= dotnum 0))(and (= len 0) (= spc 0)))
  (progn
    (princ "Invalid line definition.. Please try again..")
    (LINEDEF)
  )
)
)                                        ; End LINEDEF

;
(defun NOPOLY ()
  (princ "\nEntity selected is not a polyline, please select another ..")
  (C:REVLINE)
)
;
(defun SEGMENT ()
(while (and (= switch 1)(/= eod "YES"))
       (setq x1 (car pt1))
       (setq y1 (cadr pt1))
       (setq x2 (car pt2))                 ; Calculate distance between points
       (setq y2 (cadr pt2))
       (setq chgx (- x1 x2))
       (setq chgy (- y1 y2))
       (setq seglen (SQRT (+ (EXPT chgx 2) (EXPT chgy 2))))
       (setq dist (+ dist seglen))
       (if (< dist len)                    ; If distance less than line length
           (progn
              (command "pline" pt1 pt2 "")  ; Draw the line
              (setq dist2 dist)
              (setq pt1 pt2)               ; Set pt1 the value of pt2
              (setq ax (read-line fil))
              (setq ay (read-line fil))     ; Get next vertice values
              (if (/= ax NIL)
                  (progn                   ; If not nil, set pt2
                     (setq ax (atof ax))
                     (setq ay (atof ay))
                     (setq pt2 (list ax ay))
                  )
                  (setq eod "YES")         ; Else set end of file flag
              )
           )
           (progn                   ; if distance greater than length of line
              (setq ang (angle pt1 pt2))   ; Get angle between points
              (setq ang$ (angtos ang 0 4))
              (setq add (- len dist2))
              (setq xoff (* (cos ang) add)) ; calculate distance to draw line
              (setq yoff (* (sin ang) add))
              (setq pt3x (+ (car pt1) xoff))
              (setq pt3y (+ (cadr pt1) yoff))
              (setq pt3 (list pt3x pt3y))
              (command "pline" pt1 pt3 "")      ; Draw line
              (setq dist 0.0)
              (setq dist2 dist)                ; Reset distances
              (setq pt1 pt3)                   ; Set pt1 to value of pt3
              (setq dtcount 0)
              (setq switch 0)                    ; Set switch for Dot sequence
           )
       )
)
)
;
(defun DOT ()
(while (and (<= dtcount dotnum)(/= eod "YES"))
   (if (/= eod "YES")
      (progn
          (setq x1 (car pt1))
          (setq y1 (cadr pt1))
          (setq x2 (car pt2))        ; Calculate distance between pt1 and pt2
          (setq y2 (cadr pt2))
          (setq chgx (- x1 x2))
          (setq chgy (- y1 y2))
          (setq seglen (SQRT (+ (EXPT chgx 2) (EXPT chgy 2))))
          (setq dist (+ dist seglen))
          (if (< dist spc)
              (progn
                 (setq pt1 pt2)
                 (setq dist2 dist)
                 (setq ax (read-line fil))  ; if distance less than dot spacing
                 (setq ay (read-line fil))
                 (if (/= ax NIL)
                     (progn
                         (setq ax (atof ax))      ; get next vertex values
                         (setq ay (atof ay))
                         (setq pt2 (list ax ay))  ; set next pt2
                     )
                     (setq eod "YES")
                  )
              )
              (progn
                  (setq ang (angle pt1 pt2))
                  (setq add (- spc dist2))
                  (setq xoff (* (cos ang) add))  ; Else
                  (setq yoff (* (sin ang) add))  ; calculate distance for pt3
                  (setq pt3x (+ (car pt1) xoff))
                  (setq pt3y (+ (cadr pt1) yoff))
                  (setq pt3 (list pt3X pt3Y))
                  (if (< dtcount dotnum)
                      (command "POINT" pt3)      ; draw point
                  )
                  (setq dist 0.0)
                  (setq dist2 dist)
                  (setq pt1 pt3)                 ; set pt1 value of pt3
                  (setq dtcount (1+ dtcount))    ; increment dot count
              )
          )
      )
   )
)
(setq switch 1)           ;  Set switch for Line sequence
)
